"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertNamingConvention = void 0;
const utils_1 = require("../../utils");
function assertNamingConvention(exportValue, exportName, telefuncFilePath, appRootDir) {
    if ((0, utils_1.isProduction)())
        return;
    assertStartsWithOn(exportName, telefuncFilePath);
    assertCollocation(telefuncFilePath, appRootDir, exportValue);
}
exports.assertNamingConvention = assertNamingConvention;
function assertStartsWithOn(exportName, telefuncFilePath) {
    if (/on[A-Z]/.test(exportName))
        return;
    if (!/on/.test(exportName)) {
        (0, utils_1.assertWarning)(false, `We recommend the name of your telefunction ${exportName}() (${telefuncFilePath}) to start with "on", see https://telefunc.com/event-based#naming-convention'`, { onlyOnce: true });
    }
    else {
        (0, utils_1.assertWarning)(/on[A-Z]/.test(exportName), `The name of your telefunction ${exportName}() (${telefuncFilePath}) starts with "on" but isn't followed by a capital letter, see https://telefunc.com/event-based#naming-convention'`, { onlyOnce: true });
    }
}
function assertCollocation(telefuncFilePath, appRootDir, exportValue) {
    appRootDir = appRootDir || exportValue._appRootDir || null;
    if (!appRootDir)
        return;
    let fs;
    let path;
    const req = require;
    try {
        fs = req('fs');
        path = req('path');
    }
    catch (_a) {
        return;
    }
    const getBasename = (fileNameOrPath) => {
        (0, utils_1.assertPosixPath)(fileNameOrPath);
        let basename = path.posix.basename(fileNameOrPath).split('.')[0];
        if (basename.startsWith('+'))
            basename = basename.slice(1);
        return basename;
    };
    (0, utils_1.assertPosixPath)(telefuncFilePath);
    const telefuncFileBasename = getBasename(telefuncFilePath);
    const telefuncFileDir = path.posix.dirname(telefuncFilePath);
    const telefuncFileDirAbsolute = path.posix.join(appRootDir, telefuncFileDir);
    const collocatedFiles = fs.readdirSync(telefuncFileDirAbsolute);
    const collocatedFilesMatchYes = [];
    const collocatedFilesMatchNot = [];
    collocatedFiles.forEach((fileName) => {
        (0, utils_1.assertPosixPath)(fileName); // fileName isn't a path so it shouldn't contain any backslash windows path separator
        const fileBasename = getBasename(fileName);
        fileName = path.posix.join(telefuncFileDir, fileName);
        if (fileBasename === telefuncFileBasename) {
            collocatedFilesMatchYes.push(fileName);
        }
        else {
            collocatedFilesMatchNot.push(fileName);
        }
    });
    /* There seem to be a race condition: https://github.com/brillout/telefunc/issues/61
    assert(collocatedFilesMatchYes.length >= 1, { telefuncFilePathAbsolute, collocatedFiles })
    */
    (0, utils_1.assert)(!(0, utils_1.isProduction)()); // New lines in production logs are a no-go
    (0, utils_1.assertWarning)(collocatedFilesMatchYes.length >= 2, [
        `We recommend to collocate ${telefuncFilePath} with a UI component file, see https://telefunc.com/event-based#naming-convention`,
        '    Your telefunction:',
        `      ${telefuncFilePath} (base name: '${telefuncFileBasename}')`,
        '    Its collocated files:',
        ...collocatedFilesMatchNot.map((fileName) => `      ${fileName} (base name: '${getBasename(fileName)}'`),
        `    None of its collocated files share its base name '${telefuncFileBasename}'.`,
    ].join('\n'), { onlyOnce: true });
}
