"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleError = void 0;
const utils_1 = require("../../utils");
const globalContext_1 = require("../globalContext");
function handleError(err) {
    // We ensure we print a string; Cloudflare Workers doesn't seem to properly stringify `Error` objects.
    const errStr = ((0, utils_1.hasProp)(err, 'stack') && String(err.stack)) || String(err);
    const viteDevServer = (0, globalContext_1.getViteDevServer)();
    if (viteAlreadyLoggedError(err, viteDevServer)) {
        return;
    }
    viteErrorCleanup(err, viteDevServer);
    console.error(errStr);
}
exports.handleError = handleError;
function viteAlreadyLoggedError(err, viteDevServer) {
    if (!viteDevServer) {
        return false;
    }
    return viteDevServer.config.logger.hasErrorLogged(err);
}
function viteErrorCleanup(err, viteDevServer) {
    if (!viteDevServer) {
        return false;
    }
    if ((0, utils_1.hasProp)(err, 'stack')) {
        // Apply source maps
        viteDevServer.ssrFixStacktrace(err);
    }
}
