"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadTelefuncFilesFromConfig = void 0;
const utils_1 = require("../../utils");
const path_1 = require("path");
const import_1 = require("@brillout/import");
const picocolors_1 = __importDefault(require("@brillout/picocolors"));
const serverConfig_1 = require("../serverConfig");
async function loadTelefuncFilesFromConfig(runContext) {
    const { root, telefuncFiles } = (0, serverConfig_1.getServerConfig)();
    (0, utils_1.assertUsage)(root, `You need to set ${picocolors_1.default.cyan('config.root')} to be able to use ${picocolors_1.default.cyan('config.telefuncFiles')}, see https://telefunc.com/root`);
    (0, utils_1.assert)(telefuncFiles);
    (0, utils_1.assertPosixPath)(root);
    const telefuncFilesLoaded = {};
    const telefuncFilesAll = [];
    await Promise.all(telefuncFiles.map(async (telefuncFilePathAbsolute) => {
        const telefuncFilePath = resolveTelefuncFilePath(telefuncFilePathAbsolute, root);
        telefuncFilesAll.push(telefuncFilePath);
        (0, utils_1.assert)((0, utils_1.isTelefuncFilePath)(runContext.telefuncFilePath));
        (0, utils_1.assert)((0, utils_1.isTelefuncFilePath)(telefuncFilePath));
        if (telefuncFilePath !== runContext.telefuncFilePath) {
            return;
        }
        const telefunctions = await (0, import_1.import_)(telefuncFilePathAbsolute);
        telefuncFilesLoaded[telefuncFilePath] = telefunctions;
    }));
    return { telefuncFilesLoaded, telefuncFilesAll };
}
exports.loadTelefuncFilesFromConfig = loadTelefuncFilesFromConfig;
function resolveTelefuncFilePath(telefuncFilePathAbsolute, appRootDir) {
    (0, utils_1.assertPosixPath)(telefuncFilePathAbsolute);
    const path = path_1.posix.relative(appRootDir, telefuncFilePathAbsolute);
    (0, utils_1.assertPosixPath)(path);
    (0, utils_1.assertUsage)(!path.startsWith('../'), `The telefunc file ${telefuncFilePathAbsolute} doesn't live inside the root directory ${appRootDir} of your project. Either move your telefunc file inside the root, or change ${picocolors_1.default.cyan('config.root')} (https://telefunc.com/root).`);
    (0, utils_1.assert)(!path.startsWith('/') && !path.startsWith('.'));
    const telefuncFilePath = '/' + path;
    (0, utils_1.assert)((0, utils_1.isTelefuncFilePath)(telefuncFilePathAbsolute));
    return telefuncFilePath;
}
