"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformTelefuncFileServerSide = void 0;
const getExportNames_1 = require("./getExportNames");
const utils_1 = require("./utils");
const generateShield_1 = require("../server/shield/codegen/generateShield");
async function transformTelefuncFileServerSide(src, id, appRootDir, skipRegistration, isDev) {
    (0, utils_1.assertPosixPath)(id);
    (0, utils_1.assertPosixPath)(appRootDir);
    const exportNames = await (0, getExportNames_1.getExportNames)(src);
    let code = decorateTelefunctions(exportNames, src, id.replace(appRootDir, ''), appRootDir, skipRegistration);
    if (id.endsWith('.ts') && !isDev) {
        code = (0, generateShield_1.generateShield)(code, id, appRootDir);
    }
    return code;
}
exports.transformTelefuncFileServerSide = transformTelefuncFileServerSide;
function decorateTelefunctions(exportNames, src, filePath, appRootDir, skipRegistration) {
    (0, utils_1.assertPosixPath)(filePath);
    return [
        'import { __decorateTelefunction } from "telefunc";',
        // No break line before `src` to avoid breaking source map lines
        src,
        '\n\n',
        exportNames
            .map((exportName) => `__decorateTelefunction(${exportName}, "${exportName}", "${filePath}", "${appRootDir}", ${String(skipRegistration)});`)
            .join('\n'),
        '\n',
    ].join('');
}
