"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.install = void 0;
const path_1 = require("path");
const generateShield_1 = require("../server/shield/codegen/generateShield");
const getInfo_1 = require("./getInfo");
const dir = __dirname + (() => '')(); // trick to avoid `@vercel/ncc` to glob import
const loader = (0, path_1.resolve)(dir, './loader.js');
function install(config, logSuccessPrefix) {
    config.module.rules.push({
        test: /\.telefunc\./,
        use: [{ loader }],
    });
    if (logSuccessPrefix) {
        // Possible altenertive: use `process.on('exit', () => { /*...*/ })`
        //  - `afterEmitPlugin()` logs at the right time and fits well Next.js's build logs, but it may not fit the build logs of other stacks.
        //  - Reliable exit callback: https://stackoverflow.com/questions/14031763/doing-a-cleanup-action-just-before-node-js-exits/49392671#49392671
        config.plugins.push(afterEmitPlugin(logSuccessPrefix));
    }
}
exports.install = install;
function afterEmitPlugin(logSuccessPrefix) {
    return {
        apply: (compiler) => {
            compiler.hooks.afterEmit.tap('AfterEmitPlugin', (_compilation) => {
                const root = (0, getInfo_1.getRoot)(compiler);
                (0, generateShield_1.logResult)(root, logSuccessPrefix, null);
            });
        },
    };
}
