"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadTelefuncFiles = void 0;
const utils_1 = require("../../utils");
const loadTelefuncFilesWithVite_1 = require("../../vite/loadTelefuncFilesWithVite");
const loadTelefuncFilesWithRegistration_1 = require("./loadTelefuncFilesWithRegistration");
const loadTelefuncFilesFromConfig_1 = require("./loadTelefuncFilesFromConfig");
async function loadTelefuncFiles(runContext) {
    // Handles:
    // - When the user provides the telefunc file paths with `config.telefuncFiles`
    {
        if (runContext.telefuncFilesManuallyProvidedByUser) {
            (0, utils_1.assert)((0, utils_1.hasProp)(runContext, 'telefuncFilesManuallyProvidedByUser', 'string[]')); // Help TS narrow `runContext`
            const { telefuncFilesLoaded, telefuncFilesAll } = await (0, loadTelefuncFilesFromConfig_1.loadTelefuncFilesFromConfig)(runContext);
            (0, utils_1.assertUsage)(Object.keys(telefuncFilesAll).length > 0, getNothingFoundErr('manually provided by user'));
            return { telefuncFilesLoaded, telefuncFilesAll };
        }
    }
    // Handles:
    // - Next.js
    // - Nuxt 2
    // - Vite with `importBuild.js`
    {
        const telefuncFilesLoaded = (0, loadTelefuncFilesWithRegistration_1.loadTelefuncFilesWithRegistration)();
        if (telefuncFilesLoaded) {
            const telefuncFilesAll = Object.keys(telefuncFilesLoaded);
            (0, utils_1.assertUsage)(Object.keys(telefuncFilesAll).length > 0, getNothingFoundErr('automatic registration'));
            return { telefuncFilesLoaded, telefuncFilesAll };
        }
    }
    // Handles:
    // - Vite
    if (!(0, utils_1.isWebpack)() || (0, utils_1.isVikeApp)()) {
        const { telefuncFilesLoaded, viteProvider, telefuncFilesAll } = await (0, loadTelefuncFilesWithVite_1.loadTelefuncFilesWithVite)(runContext);
        (0, utils_1.assertUsage)(Object.keys(telefuncFilesAll).length > 0, getNothingFoundErr(`Vite [\`${viteProvider}\`]`));
        return { telefuncFilesLoaded, telefuncFilesAll };
    }
    (0, utils_1.assertUsage)(false, "You don't seem to be using Telefunc with a supported stack. Reach out on GitHub or Discord.");
}
exports.loadTelefuncFiles = loadTelefuncFiles;
function getNothingFoundErr(retrievalMethod) {
    return ('No `.telefunc.{js|ts|...}` file found. Did you create one? (Telefunc files retrieval method: ' +
        retrievalMethod +
        '.)');
}
